#include <jni.h>
#include <android/log.h>
#include "common.h"
extern "C" {
	/*
	 * Class:     jp_tonosama_komoki_janscouterUtil_JanScouterUtil
	 * Method:    decodeYUV2BIN
	 * Signature: ([BII[I)I
	 */
	JNIEXPORT jint JNICALL Java_jp_tonosama_komoki_janscouterUtil_JanScouterUtil_decodeYUV2BIN
	(JNIEnv *env, jobject me, jbyteArray img, jint imgWidth, jint imgHeight, jintArray out)
	{
		jboolean b;
		int i,j;
		int Y, Cr = 0, Cb = 0;
		int size = imgWidth * imgHeight;

		jbyte* arrImg = env->GetByteArrayElements(img, &b);
		jint*  arrOut = env->GetIntArrayElements(out,&b);

		for (j = 0; j < imgHeight; j++) {
			int pixPtr = j * imgWidth;
			int jDiv2 = j >> 1;
		for (i = 0; i < imgWidth; i++) {
			Y = arrImg[pixPtr];
			if (Y < 0) Y += 255;
			if ((i & 0x1) != 1) {
				int cOff = size + jDiv2 * imgWidth + (i >> 1) * 2;
				Cb = arrImg[cOff];
				if (Cb < 0) Cb += 127;
				else Cb -= 128;
				Cr = arrImg[cOff + 1];
				if (Cr < 0) Cr += 127;
				else Cr -= 128;
			}
			int R = Y + Cr + (Cr >> 2) + (Cr >> 3) + (Cr >> 5);
			if (R < 0) R = 0;
			else if (R > 255) R = 255;
			int G = Y - (Cb >> 2) + (Cb >> 4) + (Cb >> 5) - (Cr >> 1) + (Cr >> 3) + (Cr >> 4) + (Cr >> 5);
			if (G < 0) G = 0;
			else if (G > 255) G = 255;
			int B = Y + Cb + (Cb >> 1) + (Cb >> 2) + (Cb >> 6);
			if (B < 0) B = 0;
			else if (B > 255) B = 255;
			int value = (B + G + R) / 3;
			if( value < 128 ) B = G = R = 0;
			else              B = G = R = 255;
			arrOut[pixPtr++] = 0xff000000 + (B << 16) + (G << 8) + R;
		}
		}

		env->ReleaseByteArrayElements( img, arrImg, 0);
		env->ReleaseIntArrayElements(  out, arrOut, 0);
	}
	/*
	 * Class:     jp_tonosama_komoki_janscouterUtil_JanScouterUtil
	 * Method:    decodeYUV2GRAY
	 * Signature: ([BII[I)I
	 */
	JNIEXPORT jint JNICALL Java_jp_tonosama_komoki_janscouterUtil_JanScouterUtil_decodeYUV2GRAY
	(JNIEnv *env, jobject me, jbyteArray img, jint imgWidth, jint imgHeight, jintArray out)
	{
		jboolean b;
		int i,j;
		int Y, Cr = 0, Cb = 0;
		int size = imgWidth * imgHeight;

		jbyte* arrImg     = env->GetByteArrayElements(img, &b);
		jint*  arrOut     = env->GetIntArrayElements(out,&b);

		for (j = 0; j < imgHeight; j++) {
			int pixPtr = j * imgWidth;
			int jDiv2 = j >> 1;
		for (i = 0; i < imgWidth; i++) {
			Y = arrImg[pixPtr];
			if (Y < 0) Y += 255;
			if ((i & 0x1) != 1) {
				int cOff = size + jDiv2 * imgWidth + (i >> 1) * 2;
				Cb = arrImg[cOff];
				if (Cb < 0) Cb += 127;
				else Cb -= 128;
				Cr = arrImg[cOff + 1];
				if (Cr < 0) Cr += 127;
				else Cr -= 128;
			}
			int R = Y + Cr + (Cr >> 2) + (Cr >> 3) + (Cr >> 5);
			if (R < 0) R = 0;
			else if (R > 255) R = 255;
			int G = Y - (Cb >> 2) + (Cb >> 4) + (Cb >> 5) - (Cr >> 1) + (Cr >> 3) + (Cr >> 4) + (Cr >> 5);
			if (G < 0) G = 0;
			else if (G > 255) G = 255;
			int B = Y + Cb + (Cb >> 1) + (Cb >> 2) + (Cb >> 6);
			if (B < 0) B = 0;
			else if (B > 255) B = 255;
			int value = (B + G + R) / 3;
			B = G = R = value;
			arrOut[pixPtr++] = 0xff000000 + (B << 16) + (G << 8) + R;
		}
		}

		env->ReleaseByteArrayElements( img, arrImg, 0);
		env->ReleaseIntArrayElements(  out, arrOut, 0);
	}
	/*
	 * Class:     jp_tonosama_komoki_janscouterUtil_JanScouterUtil
	 * Method:    decodeYUV
	 * Signature: ([BII[I)I
	 */
	JNIEXPORT jint JNICALL Java_jp_tonosama_komoki_janscouterUtil_JanScouterUtil_decodeYUV2RGB
	(JNIEnv *env, jobject me, jbyteArray img, jint imgWidth, jint imgHeight, jintArray out)
	{
		jboolean b;
		int i,j;
		int Y, Cr = 0, Cb = 0;
		int size = imgWidth * imgHeight;

		jbyte* arrImg     = env->GetByteArrayElements(img, &b);
		jint*  arrOut     = env->GetIntArrayElements(out,&b);

		for (j = 0; j < imgHeight; j++) {
			int pixPtr = j * imgWidth;
			int jDiv2 = j >> 1;
		for (i = 0; i < imgWidth; i++) {
			Y = arrImg[pixPtr];
			if (Y < 0) Y += 255;
			if ((i & 0x1) != 1) {
				int cOff = size + jDiv2 * imgWidth + (i >> 1) * 2;
				Cb = arrImg[cOff];
				if (Cb < 0) Cb += 127;
				else Cb -= 128;
				Cr = arrImg[cOff + 1];
				if (Cr < 0) Cr += 127;
				else Cr -= 128;
			}
			int R = Y + Cr + (Cr >> 2) + (Cr >> 3) + (Cr >> 5);
			if (R < 0) R = 0;
			else if (R > 255) R = 255;
			int G = Y - (Cb >> 2) + (Cb >> 4) + (Cb >> 5) - (Cr >> 1) + (Cr >> 3) + (Cr >> 4) + (Cr >> 5);
			if (G < 0) G = 0;
			else if (G > 255) G = 255;
			int B = Y + Cb + (Cb >> 1) + (Cb >> 2) + (Cb >> 6);
			if (B < 0) B = 0;
			else if (B > 255) B = 255;
			arrOut[pixPtr++] = 0xff000000 + (B << 16) + (G << 8) + R;
		}
		}
		LOGV("test2");

		env->ReleaseByteArrayElements( img, arrImg, 0);
		env->ReleaseIntArrayElements(  out, arrOut, 0);
	}
	/*
	 * Class:     jp_tonosama_komoki_janscouterUtil_JanScouterUtil
	 * Method:    public static native int JanDetector(
        byte[] img,         // J擾X[摜B
        int    imgWidth,    // byte[] img ̉TCYB
        int    imgHeight,   // byte[] img ̏cTCYB
        int    guidePosX,   // KChg̊Jnʒu X W
        int    guidePosY,   // KChg̊Jnʒu Y W
        int    guideWidth,  // KChg̉TCY
        int    guideHeight, // KChg̏cTCY
        int[]  pointX,      // evXWBo͗pB
        int[]  pointY,      // evYWBo͗pB
        int[]  tileWidth,   // ev̉Bo͗pB
        int[]  tileHeight,  // ev̏cBo͗pB
        byte[] hiArray,     // ev2l摜Bo͗pB
        int[]  edgeImg      // ͉摜BfobOpB
        );
	 * Signature: ([BII[I[I[I[I)I
	 */
	JNIEXPORT jint JNICALL Java_jp_tonosama_komoki_janscouterUtil_JanScouterUtil_JanDetector
	(JNIEnv *env, jobject me, jbyteArray img, jint imgWidth, jint imgHeight, jint guidePosX, jint guidePosY, jint guideWidth, jint guideHeight, jintArray pointX, jintArray pointY, jintArray tileWidth, jintArray tileHeight, jbyteArray hiArray, jintArray edgeImg)
	{
		jboolean b;
		int i,j;
		int Y, Cr = 0, Cb = 0;
		int size = imgWidth * imgHeight;

		// Java nꂽzgp
		// ۂɔz̗vfɃANZX΁Azւ̎QƂɕϊBϊ͕ʂɗvfփANZXoB
		jbyte* arrImg        = env->GetByteArrayElements(img, &b);
		jint*  arrPointX     = env->GetIntArrayElements(pointX,&b);
		jint*  arrPointY     = env->GetIntArrayElements(pointY,&b);
		jint*  arrTileWidth  = env->GetIntArrayElements(tileWidth,&b);
		jint*  arrTileHeight = env->GetIntArrayElements(tileHeight,&b);
		jbyte* arrHiArray    = env->GetByteArrayElements(hiArray,&b);
		jint*  arrEdgeImg    = env->GetIntArrayElements(edgeImg,&b);

		unsigned char *orgImg;
		unsigned char *binaryImg;
		unsigned char *outImg;

		orgImg    = new unsigned char [guideWidth*guideHeight];
		binaryImg      = new unsigned char [guideWidth*guideHeight];
		outImg    = new unsigned char[CONST_HAI_WIDTH*CONST_HAI_HEIGHT*34];


		// zɑ΂đs -> S
		int count = 0;
		for (j = 0; j < imgHeight; j++) {
			int pixPtr = j * imgWidth;
			int jDiv2  = j >> 1;
		for (i = 0; i < imgWidth; i++) {
			Y = arrImg[pixPtr];
			if (Y < 0) Y += 255;
			if ((i & 0x1) != 1) {
				int cOff = size + jDiv2 * imgWidth + (i >> 1) * 2;
				Cb = arrImg[cOff];
				if (Cb < 0) Cb += 127;
				else Cb -= 128;
				Cr = arrImg[cOff + 1];
				if (Cr < 0) Cr += 127;
				else Cr -= 128;
			}
			int R = Y + Cr + (Cr >> 2) + (Cr >> 3) + (Cr >> 5);
			if (R < 0) R = 0;
			else if (R > 255) R = 255;
			int G = Y - (Cb >> 2) + (Cb >> 4) + (Cb >> 5) - (Cr >> 1) + (Cr >> 3) + (Cr >> 4) + (Cr >> 5);
			if (G < 0) G = 0;
			else if (G > 255) G = 255;
			int B = Y + Cb + (Cb >> 1) + (Cb >> 2) + (Cb >> 6);
			if (B < 0) B = 0;
			else if (B > 255) B = 255;

			if( j < guidePosY || (guidePosY+guideHeight) <= j || i < guidePosX || (guidePosX+guideWidth) <= i ){
			}else{
				orgImg[count]     = (R+G+B)/3;
				if(R>170 && G > 170 && B > 170){
					arrEdgeImg[count] = 255;
				}else{
					arrEdgeImg[count] = 1;
				}
				count++;
			}

			pixPtr++;
		}

		}

		// zɑ΂đs -> E
		//LOGV("test");
		//printf("AAA");

		HiStatus HiArray[36];
		int ret = 0;
		printf("%d %d\n",guideWidth,guideHeight);

		ret = extractSampleHai(guideWidth,guideHeight,orgImg,HiArray);
		printf("ret=%d",ret);
		preImageProcessForMathing(guideWidth,guideHeight,orgImg,binaryImg);
		if(ret>0){
			for(int i=0;i<36;i++){
				arrPointX[i]     = HiArray[i].Pos.x;
				arrPointY[i]     = HiArray[i].Pos.y;
				arrTileWidth[i]  = HiArray[i].Size.x;
				arrTileHeight[i] = HiArray[i].Size.y;
				//printf("i=%d %d %d %d %d",i,HiArray[i].Pos.x,HiArray[i].Pos.y,HiArray[i].Size.x,HiArray[i].Size.y);
			}
			printf("***FOUND COUNT =%d ***\n ",ret);
			arrangeHai(guideWidth,guideHeight,binaryImg,34,outImg,HiArray);
		}

		for(i=0;i<guideWidth*guideHeight;i++){
			arrEdgeImg[i] = 0xff000000 + (binaryImg[i] << 16) + (binaryImg[i] << 8) + binaryImg[i];
		}
		for(i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*34;i++){
			arrHiArray[i] = (jbyte)outImg[i];
		}

		//JNI ́Az̃Rs[đ삷̂ŁA̔zIuWFNg Java  GC ɂĉĂANative ł͎cĂ܂B
		//̂ߗpȂȂzKvB
		//ŔAz𑀍삵ʂ Java ɏ߂ǂ̃[hw肷B0 ́A߂ array B
		env->ReleaseByteArrayElements( img,        arrImg,        0);
		env->ReleaseIntArrayElements(  pointX,     arrPointX,     0);
		env->ReleaseIntArrayElements(  pointY,     arrPointY,     0);
		env->ReleaseIntArrayElements(  tileWidth,  arrTileWidth,  0);
		env->ReleaseIntArrayElements(  tileHeight, arrTileHeight, 0);
		env->ReleaseByteArrayElements( hiArray,    arrHiArray,    0);
		env->ReleaseIntArrayElements(  edgeImg,    arrEdgeImg,    0);
		delete [] outImg;
		delete [] orgImg;
		delete [] binaryImg;


		return (jint)ret;
	}
	/*
	 * Class:     jp_tonosama_komoki_janscouterUtil_JanScouterUtil
	 * Method:    public static native int JanResultDetector(
        byte[]    img,         // J擾X[摜B
        int[]     imgSize,     // byte[] img ̉TCYAcTCYAKChg̊Jn X WAY WAKChg̉TCYAcTCYB͗pB
        int[]     pointX,      // evXWBo͗pB
        int[]     pointY,      // evYWBo͗pB
        int[]     tileWidth,   // ev̉Bo͗pB
        int[]     tileHeight,  // ev̏cBo͗pB
        byte[]    hiArray,     // ev2l摜Bo͗pB
        boolean[] nakiflag,    // vtOB
        int[]     haiNum,      // vAvAhv̐Bo͗pB
        byte[]    tehaiImg,    // v̉摜B  o͗pB
        byte[]    doraImg,     // hv̉摜Bo͗pB
        byte[]    nakiImg,     // v̉摜Bo͗pB
        int[]     edgeImg      // ͉摜BfobOpB
        );
	 */
	JNIEXPORT jint JNICALL Java_jp_tonosama_komoki_janscouterUtil_JanScouterUtil_JanResultDetector
	(JNIEnv *env, jobject me,
		jbyteArray img,      jintArray imgSize,
		jintArray pointX,    jintArray pointY,
		jintArray tileWidth, jintArray tileHeight, jbyteArray hiArray,
		jbooleanArray nakiflag,
		jintArray  haiNum,
		jbyteArray tehaiImg, jbyteArray doraImg,   jbyteArray nakiImg,
		jintArray edgeImg )
	{
		jboolean b;
		int i,j;
		int Y, Cr = 0, Cb = 0;

		// Java nꂽzgp
		// ۂɔz̗vfɃANZX΁Azւ̎QƂɕϊBϊ͕ʂɗvfփANZXoB
		jbyte*    arrImg        = env->GetByteArrayElements(    img,        &b);
		jint*     arrImgSize    = env->GetIntArrayElements(     imgSize,    &b);
		jint*     arrPointX     = env->GetIntArrayElements(     pointX,     &b);
		jint*     arrPointY     = env->GetIntArrayElements(     pointY,     &b);
		jint*     arrTileWidth  = env->GetIntArrayElements(     tileWidth,  &b);
		jint*     arrTileHeight = env->GetIntArrayElements(     tileHeight, &b);
		jbyte*    arrHiArray    = env->GetByteArrayElements(    hiArray,    &b);
		jboolean* arrNakiflag   = env->GetBooleanArrayElements( nakiflag,   &b);
		jint*     arrHaiNum     = env->GetIntArrayElements(     haiNum,     &b);
		jbyte*    arrTehaiImg   = env->GetByteArrayElements(    tehaiImg,   &b);
		jbyte*    arrDoraImg    = env->GetByteArrayElements(    doraImg,    &b);
		jbyte*    arrNakiImg    = env->GetByteArrayElements(    nakiImg,    &b);
		jint*     arrEdgeImg    = env->GetIntArrayElements(     edgeImg,    &b);

		int imgWidth    = arrImgSize[0];
		int imgHeight   = arrImgSize[1];
		int guidePosX   = arrImgSize[2];
		int guidePosY   = arrImgSize[3];
		int guideWidth  = arrImgSize[4];
		int guideHeight = arrImgSize[5];
		int size = imgWidth * imgHeight;

		unsigned char *orgImg;
		unsigned char *binaryImg;
		unsigned char *tehaiHai;
		unsigned char *doraHai;
		unsigned char *nakiHai;

		orgImg    = new unsigned char [guideWidth*guideHeight];
		binaryImg = new unsigned char [guideWidth*guideHeight];


		tehaiHai = new unsigned char [CONST_HAI_WIDTH*CONST_HAI_HEIGHT*MAX_TEHAI];
    	doraHai= new unsigned char [CONST_HAI_WIDTH*CONST_HAI_HEIGHT*MAX_DORA];
   		nakiHai= new unsigned char[CONST_HAI_WIDTH*CONST_HAI_HEIGHT*MAX_NAKI];

		// =====================================================================================================================================================
		// z񑀍삱  S

		printf("******   START AGARI MAIN ********\n");

		int count = 0;
		for (j = 0; j < imgHeight; j++) {
			int pixPtr = j * imgWidth;
			int jDiv2  = j >> 1;
		for (i = 0; i < imgWidth; i++) {
			Y = arrImg[pixPtr];
			if (Y < 0) Y += 255;
			if ((i & 0x1) != 1) {
				int cOff = size + jDiv2 * imgWidth + (i >> 1) * 2;
				Cb = arrImg[cOff];
				if (Cb < 0) Cb += 127;
				else Cb -= 128;
				Cr = arrImg[cOff + 1];
				if (Cr < 0) Cr += 127;
				else Cr -= 128;
			}
			int R = Y + Cr + (Cr >> 2) + (Cr >> 3) + (Cr >> 5);
			if (R < 0) R = 0;
			else if (R > 255) R = 255;
			int G = Y - (Cb >> 2) + (Cb >> 4) + (Cb >> 5) - (Cr >> 1) + (Cr >> 3) + (Cr >> 4) + (Cr >> 5);
			if (G < 0) G = 0;
			else if (G > 255) G = 255;
			int B = Y + Cb + (Cb >> 1) + (Cb >> 2) + (Cb >> 6);
			if (B < 0) B = 0;
			else if (B > 255) B = 255;

			if( j < guidePosY || (guidePosY+guideHeight) <= j || i < guidePosX || (guidePosX+guideWidth) <= i ){
			}else{
				orgImg[count]     = (R+G+B)/3;
				arrEdgeImg[count] = 0xff000000 + (B << 16) + (G << 8) + R;;
				count++;
			}
			pixPtr++;
		}

		}

		// z񑀍삱܂  E
		// =====================================================================================================================================================
		//********************************
		//オvp
		//*****************************************

		int doraPosY=0;
		int tehaiPosY=guideHeight*2/5-10;
		int nakiPosY=guideHeight*4/5-10;
		int guideWakuY=guideHeight/4+10;

		int ret=0;
		printf("doraY=%d tehaiY=%d nakiY=%d,guideY=%d\n",doraPosY,tehaiPosY,nakiPosY,guideWakuY);
		class AgariHai agariHaiArray;
		ret= agariMain(guideWidth,guideHeight,orgImg,agariHaiArray,doraPosY,tehaiPosY,nakiPosY,guideWakuY,tehaiHai,doraHai,nakiHai);
		printf("Agari result is %d\n",ret);

		agariHaiArray.showHaiStatus();
		preImageProcess(guideWidth,guideHeight,orgImg,binaryImg);
		for(i=0;i<guideWidth*guideHeight;i++){
			arrEdgeImg[i] = 0xff000000 + (binaryImg[i] << 16) + (binaryImg[i] << 8) + binaryImg[i];
		}
		count=0;
		if(ret>0){
			for(int i=0;i<agariHaiArray.doraNum;i++){
				arrPointX[count]     = agariHaiArray.DoraArray[i].Pos.x;
				arrPointY[count]     = agariHaiArray.DoraArray[i].Pos.y+doraPosY;
				arrTileWidth[count]  = agariHaiArray.DoraArray[i].Size.x;
				arrTileHeight[count] = agariHaiArray.DoraArray[i].Size.y;
				count++;
			}
			for(int i=0;i<agariHaiArray.tehaiNum;i++){
				arrPointX[count]     = agariHaiArray.TehaiArray[i].Pos.x;
				arrPointY[count]     = agariHaiArray.TehaiArray[i].Pos.y+tehaiPosY;
				arrTileWidth[count]  = agariHaiArray.TehaiArray[i].Size.x;
				arrTileHeight[count] = agariHaiArray.TehaiArray[i].Size.y;
				count++;
			}
			for(int i=0;i<agariHaiArray.nakiNum;i++){
				arrPointX[count]     = agariHaiArray.NakiArray[i].Pos.x;
				arrPointY[count]     = agariHaiArray.NakiArray[i].Pos.y+nakiPosY;
				arrTileWidth[count]  = agariHaiArray.NakiArray[i].Size.x;
				arrTileHeight[count] = agariHaiArray.NakiArray[i].Size.y;
				//arrNakiflag[i]=agariHaiArray.NakiArray[i].feature2;
				count++;
			}


		}
		count=0;
		for(int i=0;i<agariHaiArray.doraNum;i++){
			printf("SouuterUtil_DORA:pos (%d %d) size (%d %d) \n",arrPointX[count],arrPointY[count],arrTileWidth[count],arrTileHeight[count]);
			count++;
		}
		for(int i=0;i<agariHaiArray.tehaiNum;i++){
			printf("SouuterUtil_TEHAI:pos (%d %d) size (%d %d) \n",arrPointX[count],arrPointY[count],arrTileWidth[count],arrTileHeight[count]);
			count++;
		}
		for(int i=0;i<agariHaiArray.nakiNum;i++){
			printf("SouuterUtil_NAKI:pos (%d %d) size (%d %d) \n",arrPointX[count],arrPointY[count],arrTileWidth[count],arrTileHeight[count]);
			count++;
		}



		arrHaiNum[0]=agariHaiArray.tehaiNum;
		arrHaiNum[1]=agariHaiArray.nakiNum;
		arrHaiNum[2]=agariHaiArray.doraNum;


		for(int i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*agariHaiArray.tehaiNum;i++){
			arrTehaiImg[i]=tehaiHai[i];

		}
		for(int i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*agariHaiArray.nakiNum;i++){
			arrDoraImg[i]=nakiHai[i];

		}
		for(int i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*agariHaiArray.doraNum;i++){
			arrDoraImg[i]=doraHai[i];

		}


		delete [] orgImg;
		delete [] binaryImg;
		delete [] doraHai;
		delete [] tehaiHai;
		delete [] nakiHai;
		//JNI ́Az̃Rs[đ삷̂ŁA̔zIuWFNg Java  GC ɂĉĂANative ł͎cĂ܂B
		//̂ߗpȂȂzKvB
		//ŔAz𑀍삵ʂ Java ɏ߂ǂ̃[hw肷B0 ́A߂ array B
		env->ReleaseByteArrayElements(    img,        arrImg,        0);
		env->ReleaseIntArrayElements(     imgSize,    arrImgSize,    1);
		env->ReleaseIntArrayElements(     pointX,     arrPointX,     0);
		env->ReleaseIntArrayElements(     pointY,     arrPointY,     0);
		env->ReleaseIntArrayElements(     tileWidth,  arrTileWidth,  0);
		env->ReleaseIntArrayElements(     tileHeight, arrTileHeight, 0);
		env->ReleaseByteArrayElements(    hiArray,    arrHiArray,    0);
		env->ReleaseBooleanArrayElements( nakiflag,   arrNakiflag,   0);
		env->ReleaseIntArrayElements(     haiNum,     arrHaiNum,     0);
		env->ReleaseByteArrayElements(    tehaiImg,   arrTehaiImg,   0);
		env->ReleaseByteArrayElements(    doraImg,    arrDoraImg,    0);
		env->ReleaseByteArrayElements(    nakiImg,    arrNakiImg,    0);
		env->ReleaseIntArrayElements(     edgeImg,    arrEdgeImg,    0);

		return (jint)ret;

	}
	/*
	 * Class:     jp_tonosama_komoki_janscouterUtil_JanScouterUtil
	 * Method:    public static native int JanResultMatching(
        byte[]    sampleImg, // Tvp̔v摜B͗pB
        byte[]    tehai,     // ͂v̏Bo͗pB31432239EEE ݂ȊŒlĂāAꂼusŶPvƂu\[ŶTvƂӖĂB
        byte[]    dora,      // h̏Bo͗pB
        byte[]    nakihai,   // ͂v̏Bo͗pBuvƂusY345vƂ̒lB
        boolean[] nakiflag,  // vtOB
        int[]     haiNum,    // vAvAhv̐B͗pB
	 	 haiNum[0] = v̐ haiNum[1] = v̐  haiNum[2] = hv̐
        byte[]    tehaiImg,  // v̉摜B  ͗pB
        byte[]    doraImg,   // hv̉摜B͗pB
        byte[]    nakiImg,   // v̉摜B͗pB
        int[]     edgeImg    // ͉摜BfobOpB
        );
	 */
	JNIEXPORT jint JNICALL Java_jp_tonosama_komoki_janscouterUtil_JanScouterUtil_JanResultMatching
	(JNIEnv *env, jobject me,
		jbyteArray sampleImg,
		jbyteArray tehai,    jbyteArray dora,      jbyteArray nakihai,   jbooleanArray nakiflag,
		jintArray  haiNum,
		jbyteArray tehaiImg, jbyteArray doraImg,   jbyteArray nakiImg,
		jintArray edgeImg )
	{
		jboolean b;
		int i,j;
		int Y, Cr = 0, Cb = 0;

		// Java nꂽzgp
		// ۂɔz̗vfɃANZX΁Azւ̎QƂɕϊBϊ͕ʂɗvfփANZXoB
		jbyte*    arrSampleImg  = env->GetByteArrayElements(    sampleImg,  &b);
		jbyte*    arrTehai      = env->GetByteArrayElements(    tehai,      &b);
		jbyte*    arrDora       = env->GetByteArrayElements(    dora,       &b);
		jbyte*    arrNakihai    = env->GetByteArrayElements(    nakihai,    &b);
		jboolean* arrNakiflag   = env->GetBooleanArrayElements( nakiflag,   &b);
		jint*     arrHaiNum     = env->GetIntArrayElements(     haiNum,     &b);
		jbyte*    arrTehaiImg   = env->GetByteArrayElements(    tehaiImg,   &b);
		jbyte*    arrDoraImg    = env->GetByteArrayElements(    doraImg,    &b);
		jbyte*    arrNakiImg    = env->GetByteArrayElements(    nakiImg,    &b);
		jint*     arrEdgeImg    = env->GetIntArrayElements(     edgeImg,    &b);

		unsigned char * nativeSampleImg;
		unsigned char * nativeTehaiImg;
		unsigned char * nativeDoraImg;
		unsigned char * nativeNakiImg;

		int tehaiNum=arrHaiNum[0];
		int nakiNum=arrHaiNum[1];
		int doraNum=arrHaiNum[2];


		int resultDora[MAX_DORA+1];
		int resultTehai[MAX_TEHAI+1];
		int resultNaki[MAX_NAKI+1];

		printf("MATCHING:tehaiNum=%d nakiNum=%d doraNum=%d\n",tehaiNum,nakiNum,doraNum);
		 nativeSampleImg = new unsigned char[CONST_HAI_WIDTH*CONST_HAI_HEIGHT*34];
		 nativeTehaiImg = new unsigned char[CONST_HAI_WIDTH*CONST_HAI_HEIGHT*tehaiNum];
		 nativeDoraImg  = new unsigned char[CONST_HAI_WIDTH*CONST_HAI_HEIGHT*doraNum];
		 nativeNakiImg = new unsigned char[CONST_HAI_WIDTH*CONST_HAI_HEIGHT*nakiNum];


		for(int i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*34;i++){
			nativeSampleImg[i]=arrSampleImg[i];

		}
		for(int i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*tehaiNum;i++){
			nativeTehaiImg[i]=arrTehaiImg[i];

		}
		for(int i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*doraNum;i++){
			nativeDoraImg[i]=arrDoraImg[i];

		}
		for(int i=0;i<CONST_HAI_WIDTH*CONST_HAI_HEIGHT*nakiNum;i++){
			nativeNakiImg[i]=arrNakiImg[i];

		}





		easyMatching(nativeTehaiImg,tehaiNum,nativeSampleImg,resultTehai);
		//easyMatching(nativeNakiImg,nakiNum,nativeSampleImg,resultNaki);
		//easyMatching(nativeDoraImg,doraNum,nativeSampleImg,resultDora);


			arrDora[0]=WANZU_1;
			arrDora[1]=WANZU_2;
			arrDora[2]=HAI_END;
			arrTehai[0]=PINZU_9;
			arrTehai[1]=PINZU_9;
			arrTehai[2]=HAKU;
			arrTehai[3]=HAKU;
			arrTehai[4]=HAKU;
			arrTehai[5]=HATSU;
			arrTehai[6]=HATSU;
			arrTehai[7]=HATSU;
			arrTehai[8]=TYUNN;
			arrTehai[9]=TYUNN;
			arrTehai[10]=TYUNN;
			arrTehai[11]=HAI_END;
			arrNakihai[0]=WANZU_1;
			arrNakihai[1]=WANZU_2;
			arrNakihai[2]=WANZU_3;
			arrNakihai[3]=HAI_END;
			arrNakiflag[0]=1;



		delete [] nativeSampleImg;
		delete [] nativeTehaiImg;
		delete [] nativeDoraImg;
		delete [] nativeNakiImg;


		//JNI ́Az̃Rs[đ삷̂ŁA̔zIuWFNg Java  GC ɂĉĂANative ł͎cĂ܂B
		//̂ߗpȂȂzKvB
		//ŔAz𑀍삵ʂ Java ɏ߂ǂ̃[hw肷B0 ́A߂ array B
		env->ReleaseByteArrayElements(    sampleImg,  arrSampleImg,  0);
		env->ReleaseByteArrayElements(    tehai,      arrTehai,      0);
		env->ReleaseByteArrayElements(    dora,       arrDora,       0);
		env->ReleaseByteArrayElements(    nakihai,    arrNakihai,    0);
		env->ReleaseBooleanArrayElements( nakiflag,   arrNakiflag,   0);
		env->ReleaseIntArrayElements(     haiNum,     arrHaiNum,     0);
		env->ReleaseByteArrayElements(    tehaiImg,   arrTehaiImg,   0);
		env->ReleaseByteArrayElements(    doraImg,    arrDoraImg,    0);
		env->ReleaseByteArrayElements(    nakiImg,    arrNakiImg,    0);
		env->ReleaseIntArrayElements(     edgeImg,    arrEdgeImg,    0);

		return 0;
	}
	/*
	 * Class:     jp_tonosama_komoki_janscouterUtil_JanScouterUtil
	 * Method:    public static native int JanComputeResult(
        boolean[] condition,   // corA[`AꔭAVJCzEA`J ǂB
        byte[]    tehai,       // ͂v̏B31432239EEE ݂ȊŒlĂāAꂼusŶPvƂu\[ŶTvƂӖĂB
        byte[]    dora,        // h̏B
        byte[]    nakihai,     // ͂v̏BuvƂusY345vƂ̒lB
        boolean[] nakiflag,    // vǂ̏B
        int[]     result       // tAnA_̒lBі𖼁Bo͗pB
        );
	 * Signature: ([Z[B[B[B[Z[I)I
	 */
	JNIEXPORT jint JNICALL Java_jp_tonosama_komoki_janscouterUtil_JanScouterUtil_JanComputeResult
	(JNIEnv *env, jobject me, jbooleanArray condition, jbyteArray tehai, jbyteArray dora, jbyteArray nakihai, jbooleanArray nakiflag, jintArray result)
	{
		// Java nꂽzgp
		// ۂɔz̗vfɃANZX΁Azւ̎QƂɕϊBϊ͕ʂɗvfփANZXoB
		jboolean b;
		jboolean* arrCondition = env->GetBooleanArrayElements( condition, &b);
		jbyte*    arrTehai     = env->GetByteArrayElements(    tehai,     &b);
		jbyte*    arrDora      = env->GetByteArrayElements(    dora,      &b);
		jbyte*    arrNakihai   = env->GetByteArrayElements(    nakihai,   &b);
		jboolean* arrNakiflag  = env->GetBooleanArrayElements( nakiflag,  &b);
		jint*     arrResult    = env->GetIntArrayElements(     result,    &b);

		// =====================================================================
		// vZ -> S
		//
		//fukeisan_main();


		// vZ܂ -> E
		// =====================================================================

		//JNI ́Az̃Rs[đ삷̂ŁA̔zIuWFNg Java  GC ɂĉĂANative ł͎cĂ܂B
		//̂ߗpȂȂzKvB
		//ŔAz𑀍삵ʂ Java ɏ߂ǂ̃[hw肷B0 ́A߂ array B
		env->ReleaseBooleanArrayElements( condition, arrCondition, 1);
		env->ReleaseByteArrayElements(    tehai,     arrTehai,     0);
		env->ReleaseByteArrayElements(    dora,      arrDora,      0);
		env->ReleaseByteArrayElements(    nakihai,   arrNakihai,   0);
		env->ReleaseBooleanArrayElements( nakiflag,  arrNakiflag,  1);
		env->ReleaseIntArrayElements(     result,    arrResult,    0);
	}
}
