package jp.tonosama.komoki.janscouter;

import java.util.ArrayList;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.HorizontalScrollView;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;

public class JanScouterScoreTableActivity extends Activity {

    private final String TAG = "JanScouterScoreTableActivity";
    private final boolean DebugFlg = false;

    int WC = LinearLayout.LayoutParams.WRAP_CONTENT;
    int MP = LinearLayout.LayoutParams.MATCH_PARENT;

    private int mScreenWidth;
    private int mScreenHeight;
    private float mPixelDensity;

    private LinearLayout mViewRoot;
    private ScrollView mWrapScroll;
    private HorizontalScrollView mResultScroll;

    private Button mAddingBtn;
    private Button mSettingBtn;

    int offsetX  = 0;
    int offsetY  = 0;
    int offsetScrollX = 0;
    int offsetScrollY = 0;
    int currentScrollX = 0;
    int currentScrollY = 0;
    private boolean isFirstMove = true;
    private int mScrollOrientation = 0;

    JanScouterScoreManager        mScores;
    ArrayList<ArrayList<Integer>> mScoreArray;
    ArrayList<String>             mPlayerNames;
    ArrayList<Integer>            mTotalScores;

    public JanScouterScoreTableActivity(){
        if(DebugFlg) Log.i(TAG, "Instantiated new " + this.getClass());
    }

    private void getScreenSize()
    {
        DisplayMetrics displayMetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        if(displayMetrics.widthPixels<displayMetrics.heightPixels){
        	mScreenWidth  = displayMetrics.widthPixels;
        	mScreenHeight = displayMetrics.heightPixels;;
        }else{
        	mScreenWidth  = displayMetrics.heightPixels;
        	mScreenHeight = displayMetrics.widthPixels;;
        }
        mPixelDensity = displayMetrics.density;
        Log.d(TAG, "mScreenWidth  : " + String.valueOf(mScreenWidth) );
        Log.d(TAG, "mScreenHeight : " + String.valueOf(mScreenHeight) );
        Log.d(TAG, "mPixelDensity : " + String.valueOf(mPixelDensity) );
    }

    private void findAllViewById()
    {
        ((TextView)findViewById(R.id.title)).setText( getResources().getString(R.string.title_score_result) );
        mViewRoot     = (LinearLayout)findViewById(R.id.ScoreTableRoot);
        mViewRoot.setPadding(20, 20, 20, 20);
        mSettingBtn   = (Button)findViewById(R.id.ScoreTableChgBtn);
        mAddingBtn    = (Button)findViewById(R.id.ScoreTableAddBtn);
        mWrapScroll   = (ScrollView)findViewById(R.id.ScoreTableWrapScroll);
        mResultScroll = (HorizontalScrollView)findViewById(R.id.ScoreTableScoreResult);
    }
    private void setAllViewActions()
    {
        mSettingBtn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                Intent mIntent = new Intent(getApplicationContext(),JanScouterInitialSettingActivity.class);
                startActivity(mIntent);
            }
        });
        mAddingBtn.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                mScores.addDataTest();
                setResultView();
            }
        });
/**/
        mWrapScroll.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event)
            {
            	return touchScroll(v,event);
            }
        });/**/
        mResultScroll.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event)
            {
            	return touchScroll(v,event);
            }
        });/**/
    }

    private boolean touchScroll(View v, MotionEvent event)
    {
        int x = (int) event.getRawX();
        int y = (int) event.getRawY();

        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            offsetX = x;
            offsetY = y;
            offsetScrollX  = mViewRoot.getLeft();
            currentScrollX = mViewRoot.getLeft();
            offsetScrollY  = mViewRoot.getTop();
            currentScrollY = mViewRoot.getTop();
        }
        else if (event.getAction() == MotionEvent.ACTION_MOVE) {
        	if(isFirstMove){
                offsetX = x;
                offsetY = y;
                offsetScrollX  = mViewRoot.getLeft();
                currentScrollX = mViewRoot.getLeft();
                offsetScrollY  = mViewRoot.getTop();
                currentScrollY = mViewRoot.getTop();
                isFirstMove = false;
                return true;
        	}
            int diffX = (offsetX - x);
            int diffY = (offsetY - y);
            if(mScrollOrientation==0){
            	if( Math.abs(diffX) > Math.abs(diffY) ) mScrollOrientation = 1;
            	else                                    mScrollOrientation = 2;
            }
            if(mScrollOrientation==1){
            	currentScrollX -= diffX;
            	currentScrollY -= 0;
            }else{
            	currentScrollX -= 0;
            	currentScrollY -= diffY;
            }

            int maxScrollX = mWrapScroll.getWidth()  - mViewRoot.getWidth();
            int maxScrollY = mWrapScroll.getHeight() - mViewRoot.getHeight();

            if(currentScrollX< maxScrollX) currentScrollX = maxScrollX;
            if(currentScrollX> 0)          currentScrollX = 0;

            if(currentScrollY< maxScrollY) currentScrollY = maxScrollY;
            if(currentScrollY> 0)          currentScrollY = 0;

            mViewRoot.layout(currentScrollX, currentScrollY, currentScrollX + mViewRoot.getWidth(), currentScrollY + mViewRoot.getHeight() );
            offsetX = x;
            offsetY = y;
        }
        else if (event.getAction() == MotionEvent.ACTION_UP || event.getAction() == MotionEvent.ACTION_CANCEL ) {
        	isFirstMove = true;
        	mScrollOrientation = 0;
        }
        return true;
    }

    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        //getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
        setContentView(R.layout.score_table);
        getScreenSize();
        findAllViewById();
        setAllViewActions();

        // =================================================================
        // f[^Zbg -> S
        mScores = new JanScouterScoreManager();
        mScores.setRandomPlayers();
        mScores.setRandomScores();
        // f[^Zbg -> E
        // =================================================================

        setResultView();
    }

    public void setResultView()
    {
        mViewRoot.removeAllViews();

        mScoreArray    = JanScouterScoreManager.getScoreArray();
        mPlayerNames   = JanScouterScoreManager.getPlayerNames();
        mTotalScores   = mScores.getTotalResult();
        int data_num   = mScoreArray.size();
        int player_num = mPlayerNames.size();
        // =================================================================
        // ^Cgs
        LinearLayout mTitleRow = new LinearLayout(this);
        for( int i=0;i<player_num+1;i++ )
        {
            View mSeparate = new View(this);
            LinearLayout.LayoutParams separateLP = new LinearLayout.LayoutParams(2,60);
            mSeparate.setLayoutParams(separateLP);
            mSeparate.setBackgroundColor(Color.rgb(255,255,255));
            mTitleRow.addView(mSeparate);

            TextView mScoreText = new TextView(this);
            if(i!=0) mScoreText.setText(mPlayerNames.get(i-1));
            int tabWidth;
            if(player_num<6){
            	tabWidth = (int)((mScreenWidth - 102 - 40)/(player_num))-2;
            }else{
            	tabWidth = (int)((mScreenWidth - 102 - 40)/(5))-2;
            }
            if(i==0) tabWidth = 100;
            LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(tabWidth,60);
            lp.weight  = 1;
            mScoreText.setLayoutParams(lp);
            mScoreText.setGravity(Gravity.CENTER);
            mScoreText.setTextColor(Color.BLACK);
            mTitleRow.addView(mScoreText);
        }
        mTitleRow.setBackgroundColor(Color.rgb(0, 101, 204));
        mViewRoot.addView(mTitleRow,MP,WC);

        // =================================================================
        // XRAf[^s
        LinearLayout[] mEachRowWrapper = new LinearLayout[data_num];
        for( int i=0;i<data_num;i++ )
        {
            mEachRowWrapper[i] = new LinearLayout(this);
            for( int j=0;j<player_num+1;j++ )
            {
                View mSeparate = new View(this);
                LinearLayout.LayoutParams separateLP = new LinearLayout.LayoutParams(2,60);
                mSeparate.setLayoutParams(separateLP);
                mSeparate.setBackgroundColor(Color.rgb(255,255,255));
                mEachRowWrapper[i].addView(mSeparate);

                TextView mScoreText = new TextView(this);
                if(j==0){
                    mScoreText.setText(String.format("%1$d",i+1));
                }else{
                    mScoreText.setText(String.format("%1$d", mScoreArray.get(i).get(j-1) ));
                }
                int tabWidth;
                if(player_num<6){
                	tabWidth = (int)((mScreenWidth - 102 - 40)/(player_num))-2;
                }else{
                	tabWidth = (int)((mScreenWidth - 102 - 40)/(5))-2;
                }
                if(j==0) tabWidth = 100;
                LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(tabWidth,60);
                lp.weight  = 1;
                mScoreText.setLayoutParams(lp);
                mScoreText.setGravity(Gravity.CENTER);
                mScoreText.setTextColor(Color.BLACK);
                //if(i%2==0) mScoreText.setBackgroundResource(R.drawable.table_back_1);
                //else       mScoreText.setBackgroundResource(R.drawable.table_back_2);
                /*
                if(i%2==0) mScoreText.setBackgroundColor(Color.rgb(  0, 153, 255));
                else       mScoreText.setBackgroundColor(Color.rgb(153, 204, 255));*/
                mEachRowWrapper[i].addView(mScoreText);
            }
            //
            if(i%2==0) mEachRowWrapper[i].setBackgroundResource(R.drawable.table_back_1);
            else       mEachRowWrapper[i].setBackgroundResource(R.drawable.table_back_2);
            mEachRowWrapper[i].setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {
                    Intent mIntent = new Intent(getApplicationContext(),JanScouterResultInputActivity.class);
                    startActivity(mIntent);
                }
            });
            //
            mViewRoot.addView(mEachRowWrapper[i],MP,WC);
        }

        // =================================================================
        // vs
        LinearLayout mResultRow = new LinearLayout(this);
        for( int i=0;i<player_num+1;i++ )
        {
            View mSeparate = new View(this);
            LinearLayout.LayoutParams separateLP = new LinearLayout.LayoutParams(2,60);
            mSeparate.setLayoutParams(separateLP);
            mSeparate.setBackgroundColor(Color.rgb(255,255,255));
            mResultRow.addView(mSeparate);

            TextView mScoreText = new TextView(this);
            if(i==0) mScoreText.setText("v");
            else     mScoreText.setText(String.format("%1$d", mTotalScores.get(i-1) ));
            int tabWidth;
            if(player_num<6){
            	tabWidth = (int)((mScreenWidth - 102 - 40)/(player_num))-2;
            }else{
            	tabWidth = (int)((mScreenWidth - 102 - 40)/(5))-2;
            }
            if(i==0) tabWidth = 100;
            LinearLayout.LayoutParams mResultlp = new LinearLayout.LayoutParams(tabWidth,60);
            mResultlp.weight = 1;
            mScoreText.setLayoutParams(mResultlp);
            mScoreText.setGravity(Gravity.CENTER);
            mScoreText.setTextColor(Color.BLACK);
            mResultRow.addView(mScoreText);
        }
        mResultRow.setBackgroundColor(Color.rgb(0, 101, 204));
        mViewRoot.addView(mResultRow,MP,WC);
    }
}
