package jp.tonosama.komoki.janscouter;

//import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import jp.tonosama.komoki.janscouterUtil.JanScouterPreference;
import jp.tonosama.komoki.janscouterUtil.JanScouterUtil;
import jp.tonosama.komoki.objectInfo.HiInfo;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.Bitmap.Config;
import android.graphics.Color;
import android.graphics.Matrix;
import android.hardware.Camera;
import android.hardware.SensorManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.provider.MediaStore;
import android.provider.MediaStore.Images;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.OrientationEventListener;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.Toast;

public class ResultDetectorActivity extends Activity {

    private final String TAG = "ResultDetectorActivity";
    private final boolean DebugFlg = true;

    public static final String CONDITION_KEY = "CONDITION_KEY";

    Camera mCamera;
    private ResultDetectorPreview        mPreview;
    private ResultDetectorOverlay mPreviewOverlay;
    int numberOfCameras;
    int cameraCurrentlyLocked;
    Uri imgUri = null;

    private byte[] mSampleImg;

    OrientationEventListener mOrientationListener;

    public static final int ORIENTATION_VERTICAL   = 1;
    public static final int ORIENTATION_RIGHT_SIDE = 2;
    public static final int ORIENTATION_BACK_SIDE  = 3;
    public static final int ORIENTATION_LEFT_SIDE  = 4;
    private int orientation = ORIENTATION_LEFT_SIDE;

    // Preferrence
    public boolean isOnlyAnalyze = false;
    public boolean isOnlyPreview = false;

    // The first rear facing camera
    int defaultCameraId;

    public ResultDetectorActivity(){
        if(DebugFlg) Log.i(TAG, "Instantiated new " + this.getClass());
    }
    public void getPreference(){
        isOnlyAnalyze = getIsOnlyAnalyze();
        isOnlyPreview = getIsOnlyPreview();
        mPreview.getPreference();
        mPreviewOverlay.getPreference();
    }
    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);

        // Hide the window title.
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);

        // get condition
        Bundle bundle = getIntent().getExtras();
        if( bundle!=null ){
            bundle.getBooleanArray(CONDITION_KEY);
        }else{
            finish();
        }

        // get sample hai image -> S
        int length = HiInfo.HI_WIDTH * HiInfo.HI_HEIGHT * HiInfo.TOTAL_HI_NUM ;
        String filename  = HiInfo.SAMPLE_IMAGE_FILE_NAME;

        // ============================================================================================
        // sdcard ɕۑp^[
        // fobÔߌ͂
        String directory = Environment.getExternalStorageDirectory().toString() + HiInfo.SAMPLE_IMAGE_FILE_PATH;
        FileInputStream in = null;
        File file = new File(directory, filename);
        try {
            in = new FileInputStream(file);
            mSampleImg = new byte[ length ];
            try {
                in.read(mSampleImg);
            } catch (IOException e) {
                Toast.makeText(getApplicationContext(), "IOException", Toast.LENGTH_SHORT).show();
                e.printStackTrace();
            }
        } catch (FileNotFoundException e) {
            Toast.makeText(getApplicationContext(), "FileNotFoundException", Toast.LENGTH_SHORT).show();
            e.printStackTrace();
        }

        // ============================================================================================
        // /data/data/` ɕۑp^[
        // Ƃ肠͕ۗ
        /*
        BufferedInputStream in = null;
        try {
            FileInputStream file = openFileInput(filename);
            in = new BufferedInputStream(file);
            mSampleImg = new byte[ length ];
            in.read(mSampleImg, 0, length);
            file.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            Toast.makeText(getApplicationContext(), "FileNotFoundException", Toast.LENGTH_SHORT).show();
            //finish();
        } catch(IOException e) {
            e.printStackTrace();
            Toast.makeText(getApplicationContext(), "IOException", Toast.LENGTH_SHORT).show();
            //finish();
        }*/
        // get sample hai image -> E

        mPreview = new ResultDetectorPreview(this);
        setContentView(mPreview);
        mPreviewOverlay = new ResultDetectorOverlay(this);
        addContentView( mPreviewOverlay, new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
        getPreference();
        numberOfCameras = 1;

        mOrientationListener = new OrientationEventListener(this, SensorManager.SENSOR_DELAY_UI) {
            @Override
            public void onOrientationChanged(int orientation) {
                //Toast.makeText(getApplicationContext(), "Orientation changed : " + orientation + " !", Toast.LENGTH_SHORT).show();
                mPreviewOverlay.setAngle(orientation);
                changeRotate(orientation);
            }
        };
        mOrientationListener.enable();
    }

    @Override
    protected void onResume() {
        super.onResume();

        mCamera = Camera.open();
        cameraCurrentlyLocked = defaultCameraId;
        mPreview.setCamera(mCamera);
        getPreference();
    }

    @Override
    protected void onPause() {
        super.onPause();

        // Because the Camera object is a shared resource, it's very
        // important to release it when the activity is paused.
        if (mCamera != null) {
            mPreview.setCamera(null);
            mCamera.release();
            mCamera = null;
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {

        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.camera_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        case R.id.launch_setting:
            Intent mIntent = new Intent(getApplicationContext(),JanScouterPreference.class);
            startActivity(mIntent);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }

    public ResultDetectorOverlay getPreviewOverlay(){
        return mPreviewOverlay;
    }
    public ResultDetectorPreview getCameraPreview(){
        return mPreview;
    }

    // ==============================================================================
    // if result pressed (JanScouterDetector)
    public void previewResult(HiInfo hi_info, int guide_width, int guide_height)
    {
        mPreview.mCamera.stopPreview();
        byte[]   mAllHiArray  = hi_info.getHiArray();
        byte[][] mByteHiArray = new byte[HiInfo.TOTAL_HI_NUM][HiInfo.HI_WIDTH*HiInfo.HI_HEIGHT];
        int[][]  mHiArray     = new  int[HiInfo.TOTAL_HI_NUM][HiInfo.HI_WIDTH*HiInfo.HI_HEIGHT];

        for( int i=0;i<HiInfo.TOTAL_HI_NUM;i++ ){
            for( int j=0;j<HiInfo.HI_WIDTH*HiInfo.HI_HEIGHT;j++ )
            {
                int value = mAllHiArray[i*HiInfo.HI_WIDTH*HiInfo.HI_HEIGHT + j] & 0xff;
                mHiArray[i][j]     = Color.rgb(value, value, value);
                mByteHiArray[i][j] = mAllHiArray[i*HiInfo.HI_WIDTH*HiInfo.HI_HEIGHT + j];
            }
        }
        final byte[][] rawByteArray = mByteHiArray.clone();

        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        dialog.setTitle("Result");

        ScrollView   wrapScrollView     = new ScrollView(this);
        LinearLayout resultLinearLayout = new LinearLayout(this);
        resultLinearLayout.setOrientation(LinearLayout.VERTICAL);

        Bitmap[]    hiBmp = new Bitmap[hi_info.getHiTotalNum()];
        ImageView[] image = new ImageView[hi_info.getHiTotalNum()];
        LinearLayout[] eachRowLLayout = new LinearLayout[4];

        for( int i=0;i<4;i++ )
        {
            eachRowLLayout[i] = new LinearLayout(this);
            eachRowLLayout[i].setOrientation(LinearLayout.HORIZONTAL);
        }
        for( int i=0;i<HiInfo.TOTAL_HI_NUM;i++ )
        {
            image[i] = new ImageView(this);
            hiBmp[i] = Bitmap.createBitmap(mHiArray[i], HiInfo.HI_WIDTH, HiInfo.HI_HEIGHT, Config.RGB_565);
            image[i].setImageBitmap(hiBmp[i]);
            image[i].setScaleType(ImageView.ScaleType.CENTER);
            image[i].setPadding(10, 10, 5, 5);
            eachRowLLayout[i/9].addView(image[i], ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            if(i%9==8 || i==HiInfo.TOTAL_HI_NUM-1) resultLinearLayout.addView(eachRowLLayout[i/9], ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        }
        wrapScrollView.addView(resultLinearLayout, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);

        dialog.setView(wrapScrollView);
        dialog.setPositiveButton(R.string.scouter_scan_ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton)
            {
                long dateTaken      = System.currentTimeMillis();
                String filenameBase = DateFormat.format("yyyy-MM-dd_kk.mm.ss", dateTaken).toString();
                String directory    = Environment.getExternalStorageDirectory().toString() + "/JanScouter/debug";
                OutputStream outputStream = null;

                for( int i=0;i<HiInfo.TOTAL_HI_NUM;i++ ){
                    String filename = String.format("%1$s_%2$02d.raw",filenameBase,i);
                    try {
                        File dir = new File(directory);
                        if(!dir.exists()){
                            dir.mkdirs();
                            if(DebugFlg) Log.d(TAG, dir.toString() + " create");
                        }
                        File file = new File(directory, filename);
                        if (file.createNewFile()) {
                            outputStream = new FileOutputStream(file);
                            if (rawByteArray != null) {
                                outputStream.write(rawByteArray[i]);
                            }
                        }
                    } catch (FileNotFoundException ex) {
                        Toast.makeText(getApplicationContext(), "FileNotFoundException", Toast.LENGTH_SHORT).show();
                        Log.w(TAG, ex);
                        return;
                    } catch (IOException ex) {
                        Toast.makeText(getApplicationContext(), "Please insert SD card.", Toast.LENGTH_SHORT).show();
                        Log.w(TAG, ex);
                        return;
                    } finally {
                        if (outputStream != null) {
                            try {
                                outputStream.close();
                            } catch (Throwable t) {
                            }
                        }
                    }
                }
                dialog.dismiss();
                mPreview.mCamera.startPreview();
            }
        });
        dialog.setNegativeButton(R.string.scouter_scan_ng, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton)
            {
                dialog.dismiss();
                mPreview.mCamera.startPreview();
            }
        });
        dialog.show();
    }

    // ==============================================================================
    // if result pressed (JanScouterDetector)
    public void agariResultPressed(HiInfo hi_info, int guide_width, int guide_height)
    {

    }
    // ==============================================================================
    // if bitmap pressed
    public void previewBitmap()
    {
        mPreview.mCamera.stopPreview();

        if(!isOnlyPreview){
            Bitmap bmp = mPreviewOverlay.getDecodedBitmap();
            long dateTaken   = System.currentTimeMillis();
            String name      = DateFormat.format("yyyy-MM-dd_kk.mm.ss", dateTaken).toString() + ".jpg";
            String filename  = name;
            String directory = Environment.getExternalStorageDirectory().toString() + "/" + "JanScouter";
            String filePath  = directory + "/" + filename;
            Log.d(TAG, "filepath : " + filePath);
            OutputStream outputStream = null;

            try {
                File dir = new File(directory);
                if(!dir.exists()){
                    dir.mkdirs();
                    if(DebugFlg) Log.d(TAG, dir.toString() + " create");
                }
                File file = new File(directory, filename);
                if (file.createNewFile()) {
                    outputStream = new FileOutputStream(file);
                    if (bmp != null) {
                        bmp.compress(CompressFormat.JPEG, 100, outputStream);
                    }
                }
            } catch (FileNotFoundException ex) {
                Toast.makeText(this, "FileNotFoundException", Toast.LENGTH_SHORT).show();
                Log.w(TAG, ex);
                return;
            } catch (IOException ex) {
                Toast.makeText(this, "Please insert SD card.", Toast.LENGTH_SHORT).show();
                Log.w(TAG, ex);
                return;
            } finally {
                if (outputStream != null) {
                    try {
                        outputStream.close();
                    } catch (Throwable t) {
                    }
                }
            }

            ContentValues values = new ContentValues(7);
            values.put(Images.Media.TITLE, name);
            values.put(Images.Media.DISPLAY_NAME, filename);
            values.put(Images.Media.DATE_TAKEN, dateTaken);
            values.put(Images.Media.MIME_TYPE, "image/png");
            values.put(Images.Media.DATA, filePath);
            ContentResolver cr = getContentResolver();
            imgUri = cr.insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);
        }

        Intent intent = new Intent();
        intent.setAction(Intent.ACTION_VIEW);
        intent.setType("image/jpg");
        intent.setData(imgUri);
        startActivity(intent);
    }

    // ==============================================================================
    // if preview touched
    public void touchFocus()
    {
        mPreview.autoFocus();
    }
    // ==============================================================================
    // if shutter pressed
    public void shutterPressed(boolean focusable)
    {
        if(focusable) mPreview.takePicture();
        else          mPreview.takePictureWithoutFocus();
    }

    public void saveImage()
    {
        int width    = mPreview.mPreviewSize.width;
        int height   = mPreview.mPreviewSize.height;
        byte[] mYUV  = mPreview.getCurrentYUV();
        int[] arrRGB = new int[ width * height ];
        JanScouterUtil.decodeYUV2RGB(mYUV, width, height, arrRGB);
        //JanScouterUtil.decodeYUV2GRAY(mYUV, width, height, arrRGB);
        //JanScouterUtil.decodeYUV2GRAY(mYUV, width, height, arrRGB);
        Bitmap bmp = Bitmap.createBitmap(arrRGB, width, height, Config.ARGB_8888);
        Matrix matrix = new Matrix();
        float angle = getOrientationMatrix(true);
        matrix.postRotate(angle);
        bmp = Bitmap.createBitmap(bmp, 0, 0, bmp.getWidth(), bmp.getHeight(), matrix, true);

        long dateTaken   = System.currentTimeMillis();
        String name      = DateFormat.format("yyyy-MM-dd_kk.mm.ss", dateTaken).toString() + ".jpg";
        String filename  = name;
        String directory = Environment.getExternalStorageDirectory().toString() + "/" + "JanScouter";
        String filePath  = directory + "/" + filename;
        Log.d(TAG, "filepath : " + filePath);
        OutputStream outputStream = null;

        try {
            File dir = new File(directory);
            if(!dir.exists()){
                dir.mkdirs();
                if(DebugFlg) Log.d(TAG, dir.toString() + " create");
            }
            File file = new File(directory, filename);
            if (file.createNewFile()) {
                outputStream = new FileOutputStream(file);
                if (bmp != null) {
                    bmp.compress(CompressFormat.JPEG, 100, outputStream);
                }
            }
        } catch (FileNotFoundException ex) {
            Toast.makeText(this, "FileNotFoundException", Toast.LENGTH_SHORT).show();
            Log.w(TAG, ex);
            return;
        } catch (IOException ex) {
            Toast.makeText(this, "Please insert SD card.", Toast.LENGTH_SHORT).show();
            Log.w(TAG, ex);
            return;
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (Throwable t) {
                }
            }
        }

        ContentValues values = new ContentValues(7);
        values.put(Images.Media.TITLE, name);
        values.put(Images.Media.DISPLAY_NAME, filename);
        values.put(Images.Media.DATE_TAKEN, dateTaken);
        values.put(Images.Media.MIME_TYPE, "image/png");
        values.put(Images.Media.DATA, filePath);
        ContentResolver cr = getContentResolver();
        imgUri = cr.insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);

        mPreviewOverlay.setDecodeBitmap(bmp);
        mPreviewOverlay.invalidate();
    }

    // ==============================================================================
    // invalidate Overlay
    public void invalidateOverlay(){
        mPreviewOverlay.invalidate();
    }
    // ==============================================================================
    // if rotate pressed
    public void changeRotate(int degree){
        if(   0 <= degree && degree <  45 )
            orientation = ORIENTATION_VERTICAL;
        if(  45 <= degree && degree < 135 )
            orientation = ORIENTATION_RIGHT_SIDE;
        if( 135 <= degree && degree < 225 )
            orientation = ORIENTATION_BACK_SIDE;
        if( 225 <= degree && degree < 315 )
            orientation = ORIENTATION_LEFT_SIDE;
        if( 315 <= degree && degree < 360 )
            orientation = ORIENTATION_VERTICAL;
    }
    public float getOrientationMatrix(boolean is_reverse){
        float angle = 0f;
        switch(orientation){
        case ResultDetectorActivity.ORIENTATION_VERTICAL:
            if(is_reverse) angle =  90f;
            else           angle = -90f;
            break;
        case ResultDetectorActivity.ORIENTATION_RIGHT_SIDE:
            angle = 180f;
            break;
        case ResultDetectorActivity.ORIENTATION_BACK_SIDE:
            if(is_reverse) angle = -90f;
            else           angle =  90f;
            break;
        case ResultDetectorActivity.ORIENTATION_LEFT_SIDE:
        default:
            break;
        }
        return angle;
    }
    // ==============================================================================
    // get SharedPreference
    public boolean getIsOnlyAnalyze(){
        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
        return pref.getBoolean("IS_ONLY_ANALYZE", false);
    }
    public boolean getIsOnlyPreview(){
        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
        return pref.getBoolean("IS_ONLY_PREVIEW", false);
    }
    // ==============================================================================
    // get the sample hai image byteArray
    public byte[] getSampleHaiArray(){
        return mSampleImg;
    }
}