package jp.tonosama.komoki.janscouter;

import jp.tonosama.komoki.objectInfo.HiInfo;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Rect;
import android.graphics.Paint.Style;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

public class ResultDetectorOverlay extends View {

    private final String TAG = "ResultDetectorOverlay";
    private final boolean DebugFlg = true;

    private Context mContext;

    private String mFps = "0";
    private int    mDegree;
    private long   mDetectedTime = 0;
    private Bitmap mDecodeBitmap = null;  //
    private Bitmap mShutterNormalBitmap;  //
    private Bitmap mShutterPressedBitmap; //
    private Bitmap mRotateNormalBitmap;   //
    private Bitmap mRotatePressedBitmap;  //

    private HiInfo mHiInfo = null;

    private float width, height;
    private static final int HANDLER_MESSAGE_ON_SHUTTER    = 1;
    private static final int HANDLER_MESSAGE_ON_PREVIEW    = 2;
    private static final int HANDLER_MESSAGE_CHANGE_ROTATE = 3;
    private static final int HANDLER_MESSAGE_TOUCH_FOCUS   = 4;
    private static final int HANDLER_MESSAGE_TOUCH_RESULT  = 5;

    private int guidePosX;
    private int guidePosY;
    private int guideWidth;
    private int guideHeight;

    private int doraPosX;
    private int doraPosY;
    private int doraWidth;
    private int doraHeight;

    private int tehaiPosX;
    private int tehaiPosY;
    private int tehaiWidth;
    private int tehaiHeight;

    private int nakiPosX;
    private int nakiPosY;
    private int nakiWidth;
    private int nakiHeight;

    private int focusX;
    private int focusY;
    private long focusStartTime = 0;

    private boolean isShutterPressed = false;
    private boolean isRotatePressed  = false;
    private boolean isBitmapPressed  = false;
    private boolean isPreviewPressed = false;
    private boolean isResultPressed  = false;
    //private boolean isFocusedFlag    = false;

    private Paint boardPaint;
    private Paint bmpTouchPaint;
    private Paint resultTouchPaint;
    private Paint guidePaint;
    private Paint detectPaint;
    private Paint focusPaint;
    private Paint debugTextPaint;
    private Paint detectTextPaint;

    private final float guideStrokeWidth = 10f;

    private Rect  mSttRect;
    private Rect  mEndRect;

    // Preferrence
    @SuppressWarnings("unused")
    private boolean isOnlyAnalyze = false;
    private boolean isOnlyPreview = false;

    public ResultDetectorOverlay(Context context) {
        super(context);
        if(DebugFlg) Log.i(TAG, "Instantiated new " + this.getClass());

        mContext = context;
        Resources r = getResources();
        mShutterNormalBitmap  = BitmapFactory.decodeResource(r, R.drawable.btn_camera_shutter_holo);
        mShutterPressedBitmap = BitmapFactory.decodeResource(r, R.drawable.btn_camera_shutter_pressed_holo);
        //mRotateNormalBitmap   = BitmapFactory.decodeResource(r, R.drawable.btn_retake_shutter_holo);
        //mRotatePressedBitmap  = BitmapFactory.decodeResource(r, R.drawable.btn_retake_shutter_pressed_holo);
        mRotateNormalBitmap   = BitmapFactory.decodeResource(r, R.drawable.ic_launcher_info_normal_holo);
        mRotatePressedBitmap  = BitmapFactory.decodeResource(r, R.drawable.ic_launcher_info_active_holo);

        // Vb^[{^̔wi
        boardPaint = new Paint();
        boardPaint.setStyle(Paint.Style.FILL);
        boardPaint.setColor(Color.DKGRAY);
        //boardPaint.setAlpha(128);
        boardPaint.setAlpha(255);

        // ~jvr[^b`̔wi
        bmpTouchPaint = new Paint();
        bmpTouchPaint.setStyle(Paint.Style.FILL);
        bmpTouchPaint.setColor(Color.CYAN);
        bmpTouchPaint.setAlpha(128);

        // oʃ^b`̔wi
        resultTouchPaint = new Paint();
        resultTouchPaint.setStyle(Paint.Style.FILL);
        resultTouchPaint.setColor(Color.YELLOW);
        resultTouchPaint.setAlpha(128);

        // KChg
        guidePaint = new Paint();
        guidePaint.setColor(Color.CYAN);
        guidePaint.setStyle(Style.STROKE);

        // o̘g
        detectPaint = new Paint();
        detectPaint.setColor(Color.RED);
        detectPaint.setStrokeWidth(5f);
        detectPaint.setStyle(Style.STROKE);

        // tH[JXg
        focusPaint = new Paint();
        focusPaint.setColor(Color.WHITE);
        focusPaint.setStrokeWidth(2f);
        focusPaint.setStyle(Style.STROKE);

        // fobOp̃eLXg
        debugTextPaint = new Paint();
        debugTextPaint.setAntiAlias(true);
        debugTextPaint.setColor(Color.CYAN);
        debugTextPaint.setShadowLayer(5, 1, 1, Color.BLACK);

        // oʂ̃eLXg
        detectTextPaint = new Paint();
        detectTextPaint.setAntiAlias(true);
        detectTextPaint.setColor(Color.RED);
        detectTextPaint.setShadowLayer(5, 1, 1, Color.BLACK);

        mSttRect = new Rect();
        mEndRect = new Rect();
    }
    public void getPreference(){
        isOnlyAnalyze = ((ResultDetectorActivity)mContext).getIsOnlyAnalyze();
        isOnlyPreview = ((ResultDetectorActivity)mContext).getIsOnlyPreview();
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        width  = w;
        height = h;
    }

    public void setFps(String fps) {
        mFps = fps;
    }
    public void setAngle(int orientation){
    	mDegree = orientation;
    }
    public void setDecodeBitmap(Bitmap decode_bitmap){
        mDecodeBitmap = decode_bitmap;
    }
    public void setHiInfo(HiInfo hi_info){
        mHiInfo = hi_info;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        canvas.drawColor(Color.TRANSPARENT);

        // ==================================================================
        // ȍꍇ
        if(mHiInfo!=null){
            if(mHiInfo.getDetectResult() > 6){
                mDetectedTime = System.currentTimeMillis();
            }
        }

        Matrix matrix = new Matrix();
        float angle = ((ResultDetectorActivity)mContext).getOrientationMatrix(false);
        matrix.postRotate(angle);
        // ==================================================================
        // ʂ̃CAEg` -> S
        float mScale        = 1.0f;
        float screenWidth   = width;
        float screenHeight  = height;
        float buttonWidth   = mShutterNormalBitmap.getWidth() /mScale;
        float buttonHeight  = mShutterNormalBitmap.getHeight()/mScale;
        float shadowWidth   = buttonWidth * 1.333f;
        @SuppressWarnings("unused")
        float shadowHeight  = screenHeight;
        float previewWidth  = screenWidth - shadowWidth;
        float previewHeight = screenHeight;
        // ʂ̃CAEg` -> E
        // ==================================================================
        // TCY̒` -> S
        debugTextPaint.setTextSize(height/24);
        detectTextPaint.setTextSize(height/12);
        // TCY̒` -> E
        // ==================================================================
        // KCḧʒu` -> S
        guideWidth  = (int)(previewWidth * 0.88f);
        guideHeight = (int)(guideWidth / 1.65f);
        guidePosX   = (int)(previewWidth / 2 - guideWidth *1/2);
        guidePosY   = (int)(height       / 2 - guideHeight*1/2);
        // KCḧʒu` -> E
        // ==================================================================
        // ḧʒu` -> S
        doraWidth  = (int)(guideWidth  * 0.95f);
        doraHeight = (int)(guideHeight * 0.95f / 3f);
        doraPosX   = (int)(previewWidth / 2);
        doraPosY   = (int)(height       / 2 - doraHeight);
        // ḧʒu` -> E
        // ==================================================================
        // v̈ʒu` -> S
        tehaiWidth  = (int)(guideWidth  * 0.95f);
        tehaiHeight = (int)(guideHeight * 0.95f / 3f);
        tehaiPosX   = (int)(previewWidth / 2);
        tehaiPosY   = (int)(height       / 2);
        // v̈ʒu` -> E
        // ==================================================================
        // v̈ʒu` -> S
        nakiWidth  = (int)(guideWidth  * 0.95f);
        nakiHeight = (int)(guideHeight * 0.95f / 3f);
        nakiPosX   = (int)(previewWidth / 2);
        nakiPosY   = (int)(height       / 2 + nakiHeight);
        // v̈ʒu` -> E
        // ==================================================================

        // Vb^[{^̔wi
        canvas.drawRect( screenWidth - shadowWidth, 0, screenWidth, screenHeight, boardPaint);

        // fobOp\ -> S
        debugTextPaint.setColor(Color.CYAN);
        canvas.drawText("FPS: "+mFps, 0, Math.abs(debugTextPaint.getFontMetrics().ascent)*1.2f, debugTextPaint);
        if( isOnlyPreview ){
            canvas.drawText("Detection: OFF", 0, Math.abs(debugTextPaint.getFontMetrics().ascent)*2.4f, debugTextPaint);
        }else{
            canvas.drawText("Detection: ON",  0, Math.abs(debugTextPaint.getFontMetrics().ascent)*2.4f, debugTextPaint);
        }
        canvas.drawText("Degree: "+mDegree, 0, Math.abs(debugTextPaint.getFontMetrics().ascent)*3.6f, debugTextPaint);
        // fobOp\ -> E

        // tH[JXg̕\ -> S
        if(focusStartTime + 1000 > System.currentTimeMillis() ){
            float focus_len = screenHeight / 6;
            canvas.drawRect(
                    focusX - focus_len*1/2,
                    focusY - focus_len*1/2,
                    focusX + focus_len*1/2,
                    focusY + focus_len*1/2, focusPaint );
        }
        // tH[JXg̕\ -> E

        // Decoded Bitmap ̕\ -> S
        if( mDecodeBitmap!=null ){
            Bitmap mMiniPreview = Bitmap.createBitmap(mDecodeBitmap,0,0, mDecodeBitmap.getWidth(), mDecodeBitmap.getHeight(), matrix, true);
            if(mMiniPreview.getHeight()>mMiniPreview.getWidth()){
                mSttRect.set(
                        0,
                        (mMiniPreview.getHeight()-mMiniPreview.getWidth())/2,
                        mMiniPreview.getWidth(),
                        (mMiniPreview.getHeight()-mMiniPreview.getWidth())/2 + mMiniPreview.getWidth());
            }else{
                mSttRect.set(
                        (mMiniPreview.getWidth()-mMiniPreview.getHeight())/2,
                        0,
                        (mMiniPreview.getWidth()-mMiniPreview.getHeight())/2 + mMiniPreview.getHeight(),
                        mMiniPreview.getHeight());
            }
            mEndRect.set(
                    (int)( previewWidth + shadowWidth/2 - buttonWidth/2 ),
                    (int)( screenHeight*5/6 - buttonHeight/2 ),
                    (int)( previewWidth + shadowWidth/2 + buttonWidth/2 ),
                    (int)( screenHeight*5/6 + buttonHeight/2 ));
            canvas.drawBitmap(mMiniPreview,mSttRect,mEndRect, null);
            if( isBitmapPressed ){
                canvas.drawRect(mEndRect, bmpTouchPaint);
            }
        }
        // Decoded Bitmap ̕\ -> E

        // Vb^[{^ ̕\ -> S
        Bitmap mShutterBitmap = ( isShutterPressed ) ?
                Bitmap.createBitmap(mShutterPressedBitmap,0,0, mShutterPressedBitmap.getWidth(), mShutterPressedBitmap.getHeight(), matrix, true) :
                Bitmap.createBitmap(mShutterNormalBitmap,0,0,  mShutterNormalBitmap.getWidth(),  mShutterNormalBitmap.getHeight(),  matrix, true) ;
        mSttRect.set(0,0,mShutterBitmap.getWidth(),mShutterBitmap.getHeight());
        mEndRect.set(
                (int)( previewWidth + shadowWidth/2 - buttonWidth/2 ),
                (int)( screenHeight*1/2 - buttonHeight/2 ),
                (int)( previewWidth + shadowWidth/2 + buttonWidth/2 ),
                (int)( screenHeight*1/2 + buttonHeight/2 ));
        canvas.drawBitmap(mShutterBitmap,mSttRect,mEndRect, null);
        // Vb^[ ̕\ -> E

        // ]{^ ̕\ -> S
        Matrix mMatrix = new Matrix();
        mMatrix.postRotate(-mDegree-90);
        if(!isOnlyPreview) mMatrix = matrix;

        Bitmap mRotateBitmap = ( isRotatePressed ) ?
            Bitmap.createBitmap(mRotatePressedBitmap,0,0, mRotatePressedBitmap.getWidth(), mRotatePressedBitmap.getHeight(), mMatrix, true) :
            Bitmap.createBitmap(mRotateNormalBitmap,0,0,  mRotateNormalBitmap.getWidth(),  mRotateNormalBitmap.getHeight(),  mMatrix, true) ;
        mSttRect.set(0,0,mRotateBitmap.getWidth(),mRotateBitmap.getHeight());
        mEndRect.set(
                (int)( previewWidth + shadowWidth/2 - mRotateBitmap.getWidth()/2 ),
                (int)( screenHeight*1/6 - mRotateBitmap.getHeight()/2 ),
                (int)( previewWidth + shadowWidth/2 + mRotateBitmap.getWidth()/2 ),
                (int)( screenHeight*1/6 + mRotateBitmap.getHeight()/2 ));
        canvas.drawBitmap(mRotateBitmap,mSttRect,mEndRect, null);
        // ]{^ ̕\ -> E

        if(isOnlyPreview) return;

        // v̒Z`\ -> S
        if( mHiInfo!=null &&
           ( mHiInfo.getDetectResult() > 0 ) ){
            for(int i=0;i<mHiInfo.getHiTotalNum();i++){
                float pos_x = guidePosX + mHiInfo.getHiPoint().get(i).x;
                float pos_y = guidePosY + mHiInfo.getHiPoint().get(i).y;
                float hi_width  = 1 / mScale * mHiInfo.getHiSize().get(i).x;
                float hi_height = 1 / mScale * mHiInfo.getHiSize().get(i).y;
                canvas.drawRect(
                        pos_x ,
                        pos_y ,
                        pos_x + hi_width,
                        pos_y + hi_height, detectPaint );
            }
        }
        // v̒Z`\ -> E

        // vop̃KCh -> S
        guidePaint.setStrokeWidth(guideStrokeWidth/mScale);
        guidePaint.setColor(Color.RED);
        if(mHiInfo!=null && mHiInfo.getDetectResult() > 6)
        {
            guidePaint.setColor(Color.YELLOW);
            debugTextPaint.setColor(Color.YELLOW);
            detectTextPaint.setColor(Color.YELLOW);
        }
        canvas.drawRect(
                previewWidth / 2 - guideWidth *1/2 * 0.95f,
                height       / 2 - guideHeight*1/2 * 0.95f,
                previewWidth / 2 + guideWidth *1/2 * 0.95f,
                height       / 2 + guideHeight*1/2 * 0.95f, guidePaint );
        // vop̃KCh -> E

        // voʂ̕\ -> S
        if(mHiInfo!=null){
            float textWidth  = detectTextPaint.measureText(mHiInfo.getDetectResultMsg());
            float textHeight = detectTextPaint.getFontMetrics().ascent;
            debugTextPaint.setColor(Color.RED);
            canvas.drawText(mHiInfo.getDetectResultMsg(),
                    previewWidth  / 2 - textWidth  / 2,
                    previewHeight / 2 - textHeight / 2, detectTextPaint);
        }
        // voʂ̕\ -> E

        // h/v/vop̃KCh -> S
        debugTextPaint.setColor(Color.BLUE);
        detectTextPaint.setColor(Color.BLUE);

        // fobOp\ -> S
        guidePaint.setColor(Color.BLUE);
        canvas.drawRect(
                doraPosX  - (doraWidth  / 2 * 0.95f),
                doraPosY  - (doraHeight / 2 * 0.80f),
                doraPosX  + (doraWidth  / 2 * 0.95f),
                doraPosY  + (doraHeight / 2 * 0.80f), guidePaint );
        canvas.drawRect(
                tehaiPosX - (tehaiWidth  / 2 * 0.95f),
                tehaiPosY - (tehaiHeight / 2 * 0.80f),
                tehaiPosX + (tehaiWidth  / 2 * 0.95f),
                tehaiPosY + (tehaiHeight / 2 * 0.80f), guidePaint );
        canvas.drawRect(
                nakiPosX  - (nakiWidth  / 2 * 0.95f),
                nakiPosY  - (nakiHeight / 2 * 0.80f),
                nakiPosX  + (nakiWidth  / 2 * 0.95f),
                nakiPosY  + (nakiHeight / 2 * 0.80f), guidePaint );
        detectTextPaint.setColor(Color.rgb(255, 160, 0));
        canvas.drawText(
                "DORA",
                doraPosX  - (doraWidth  / 2 * 0.95f),
                doraPosY  - (doraHeight / 2 * 0.80f), detectTextPaint);
        canvas.drawText(
                "TEHAI",
                tehaiPosX - (tehaiWidth  / 2 * 0.95f),
                tehaiPosY - (tehaiHeight / 2 * 0.80f),detectTextPaint);
        canvas.drawText(
                "NAKI",
                nakiPosX  - (nakiWidth  / 2 * 0.95f),
                nakiPosY  - (nakiHeight / 2 * 0.80f), detectTextPaint);
        // h/v/vop̃KCh -> E

    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        int posX = (int)event.getX();
        int posY = (int)event.getY();

        int sttShutterX,sttShutterY,endShutterX,endShutterY;
        int sttRotateX,sttRotateY,endRotateX,endRotateY;
        int sttGuideX = 0,sttGuideY = 0,endGuideX = 0,endGuideY = 0;
        int sttBitmapX = 0,sttBitmapY = 0,endBitmapX = 0,endBitmapY = 0;

        switch (event.getAction()) {
        case MotionEvent.ACTION_DOWN:

        	((ResultDetectorActivity)mContext).getCameraPreview().setIsUIoperation(true);

            float mScale        = 1.0f;
            float screenWidth   = width;
            float screenHeight  = height;
            float buttonWidth   = mShutterNormalBitmap.getWidth() /mScale;
            float buttonHeight  = mShutterNormalBitmap.getHeight()/mScale;
            float shadowWidth   = buttonWidth * 1.333f;
            //float shadowHeight  = screenHeight;
            float previewWidth  = screenWidth - shadowWidth;
            //float previewHeight = screenHeight;

            sttShutterX = (int)(previewWidth + shadowWidth/2 - buttonWidth/2);
            endShutterX = (int)(previewWidth + shadowWidth/2 + buttonWidth/2);
            sttShutterY = (int)(screenHeight*1/2 - buttonHeight/2);
            endShutterY = (int)(screenHeight*1/2 + buttonHeight/2);

            sttRotateX  = (int)(previewWidth + shadowWidth/2 - buttonWidth/2);
            endRotateX  = (int)(previewWidth + shadowWidth/2 + buttonWidth/2);
            sttRotateY  = (int)(screenHeight*1/6 - buttonHeight/2);
            endRotateY  = (int)(screenHeight*1/6 + buttonHeight/2);

            //if( mHiInfo!=null && mHiInfo.getDetectResult()==HiInfo.RESULT_DETECT_OK ){
            if( mDetectedTime + 5000 > System.currentTimeMillis() )
            {
                sttGuideX   = (int)(previewWidth / 2 - guideWidth *1/2 * 0.95f);
                sttGuideY   = (int)(height       / 2 - guideHeight*1/2 * 0.95f);
                endGuideX   = (int)(previewWidth / 2 + guideWidth *1/2 * 0.95f);
                endGuideY   = (int)(height       / 2 + guideHeight*1/2 * 0.95f);
            }

            if( mDecodeBitmap!=null ){
                sttBitmapX = (int)(previewWidth + shadowWidth/2 - buttonWidth/2);
                endBitmapX = (int)(previewWidth + shadowWidth/2 + buttonWidth/2);
                sttBitmapY = (int)(screenHeight*5/6 - buttonHeight/2);
                endBitmapY = (int)(screenHeight*5/6 + buttonHeight/2);
            }

            if( sttShutterX <= posX && posX <= endShutterX && sttShutterY <= posY && posY <= endShutterY ){
                isShutterPressed = true;
                this.invalidate();
            }
            else if( sttRotateX <= posX && posX <= endRotateX && sttRotateY <= posY && posY <= endRotateY ){
                isRotatePressed = true;
                this.invalidate();
            }
            else if( sttBitmapX <= posX && posX <= endBitmapX && sttBitmapY <= posY && posY <= endBitmapY ){
                isBitmapPressed = true;
                this.invalidate();
            }
            else if( sttGuideX <= posX && posX <= endGuideX && sttGuideY <= posY && posY <= endGuideY ){
                isResultPressed = true;
                this.invalidate();
            }
            else{
                isPreviewPressed = true;
            }

            if(DebugFlg) Log.d(TAG, "touch down (" + posX + "," + posY + ") ShutterBtn judge " + sttShutterX + "=<x=<" + endShutterX + " , " + sttShutterY + "=<y=<" + endShutterY + "" );
            if(DebugFlg) Log.d(TAG, "touch down (" + posX + "," + posY + ") RotateBtn  judge " + sttRotateX + "=<x=<" + endRotateX + " , " + sttRotateY + "=<y=<" + endRotateY + "" );
            if(DebugFlg) Log.d(TAG, "touch down (" + posX + "," + posY + ") BitmapBtn  judge " + sttBitmapX + "=<x=<" + endBitmapX + " , " + sttBitmapY + "=<y=<" + endBitmapY + "" );

            break;

        case MotionEvent.ACTION_UP:
            if( isRotatePressed ){
                handler.sendEmptyMessage(HANDLER_MESSAGE_CHANGE_ROTATE);
                isRotatePressed  = false;
                this.invalidate();
            }
            if(isResultPressed){
                handler.sendEmptyMessage(HANDLER_MESSAGE_TOUCH_RESULT);
                isResultPressed = false;
                this.invalidate();
            }
            if( isShutterPressed ){
                handler.sendEmptyMessage(HANDLER_MESSAGE_ON_SHUTTER);
                isShutterPressed = false;
                this.invalidate();
            }
            if( isBitmapPressed ){
                handler.sendEmptyMessage(HANDLER_MESSAGE_ON_PREVIEW);
                isBitmapPressed = false;
                this.invalidate();
            }
            if( isPreviewPressed ){
                handler.sendEmptyMessage(HANDLER_MESSAGE_TOUCH_FOCUS);
                focusStartTime = System.currentTimeMillis();
                focusX = posX;
                focusY = posY;
                isPreviewPressed = false;
            }
            ((ResultDetectorActivity)mContext).getCameraPreview().setIsUIoperation(false);
            break;
        }
        return true;
    }

    private final Handler handler = new Handler() {
        public void handleMessage(Message msg) {
            switch(msg.what){
            case HANDLER_MESSAGE_ON_SHUTTER:
                ((ResultDetectorActivity)mContext).shutterPressed(false);
                break;
            case HANDLER_MESSAGE_TOUCH_RESULT:
                ((ResultDetectorActivity)mContext).previewResult(mHiInfo,guideWidth,guideHeight);
                break;
            case HANDLER_MESSAGE_ON_PREVIEW:
                ((ResultDetectorActivity)mContext).previewBitmap();
                break;
            case HANDLER_MESSAGE_CHANGE_ROTATE:
                //((ResultDetectorActivity)mContext).changeRotate(!((ResultDetectorActivity)mContext).getIsVertical());
                break;
            case HANDLER_MESSAGE_TOUCH_FOCUS:
                ((ResultDetectorActivity)mContext).touchFocus();
                break;
            }
        }
    };

    // ========================================================================
    // Draw Object ̏擾
    public Bitmap getDecodedBitmap(){
        return mDecodeBitmap;
    }
    public Point[] getGuidePointInfo()
    {
        Point guidePos  = new Point(guidePosX,guidePosY);
        Point guideSize = new Point(guideWidth,guideHeight);
        Point[] guideInfo = {guidePos,guideSize};
        return guideInfo;
    }
}