package jp.tonosama.komoki.janscouter;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.ImageFormat;
import android.graphics.Bitmap.Config;
import android.graphics.Matrix;
import android.graphics.Point;
import android.hardware.Camera;
import android.hardware.Camera.PreviewCallback;
import android.hardware.Camera.Size;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;

import java.io.IOException;
import java.util.List;

import jp.tonosama.komoki.janscouterUtil.JanScouterUtil;
import jp.tonosama.komoki.objectInfo.HiInfo;

public class ResultDetectorPreview extends ViewGroup implements SurfaceHolder.Callback, Runnable {

    private final String TAG = "ResultDetectorPreview";
    private final boolean DebugFlg = true;

    SurfaceView mSurfaceView;
    SurfaceHolder mHolder;
    Size mPreviewSize;
    List<Size> mSupportedPreviewSizes;
    Camera mCamera;
    private Context mContext;
    private byte[]   mBuffer;
    private byte[]   mFrame;
    private boolean  mThreadRun;

    private Bitmap bmp;
    private String fps = "";
    private byte[] mYUV        = null;
    private byte[] sampleImg   = null;
    private byte[] tehai       = null;
    private byte[] dora        = null;
    private byte[] nakihai     = null;
    private boolean[] nakiflag = null;
    private int[]  haiNum      = null;
    private byte[] tehaiImg    = null;
    private byte[] doraImg     = null;
    private byte[] nakiImg     = null;
    private int[] edgeImg      = null;

    // for JanDetector
    private HiInfo  mHiInfo    = null;
    private Point[] mGuideInfo = null;
    private byte[]  mHiArray   = null;
    private int[]   mHiPointX  = null;
    private int[]   mHiPointY  = null;
    private int[]   mHiWidth   = null;
    private int[]   mHiHeight  = null;

    //
    private boolean inUIoperation = false;

    // Preferrence
    public boolean isOnlyAnalyze = false;
    public boolean isOnlyPreview = false;

    //
    private final int AGARI_RESULT_NG = -1;
    private final int AGARI_RESULT_OK = 1;

    ResultDetectorPreview(Context context) {
        super(context);
        if(DebugFlg) Log.i(TAG, "Instantiated new " + this.getClass());

        mContext = context;
        mSurfaceView = new SurfaceView(context);
        addView(mSurfaceView);

        mHolder = mSurfaceView.getHolder();
        mHolder.addCallback(this);
        mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);

        mHiInfo   = new HiInfo();
        mHiPointX = new int[HiInfo.SCAN_HI_NUM];
        mHiPointY = new int[HiInfo.SCAN_HI_NUM];
        mHiWidth  = new int[HiInfo.SCAN_HI_NUM];
        mHiHeight = new int[HiInfo.SCAN_HI_NUM];
        mGuideInfo = new Point[2];
        mHiArray  = new byte[HiInfo.HI_WIDTH*HiInfo.HI_HEIGHT*HiInfo.TOTAL_HI_NUM];

        sampleImg = ((ResultDetectorActivity)context).getSampleHaiArray();
    }
    public void getPreference(){
        isOnlyAnalyze = ((ResultDetectorActivity)mContext).getIsOnlyAnalyze();
        isOnlyPreview = ((ResultDetectorActivity)mContext).getIsOnlyPreview();
    }
    public void setCamera(Camera camera) {
        mCamera = camera;
        if (mCamera != null) {
            mSupportedPreviewSizes = mCamera.getParameters().getSupportedPreviewSizes();
            requestLayout();

            mCamera.setPreviewCallbackWithBuffer(new PreviewCallback() {
                public void onPreviewFrame(byte[] data, Camera camera) {
                    synchronized (ResultDetectorPreview.this) {
                        System.arraycopy(data, 0, mFrame, 0, data.length);
                        ResultDetectorPreview.this.notify();
                    }
                    camera.addCallbackBuffer(mBuffer);
                }
            });
        }
    }

    public void switchCamera(Camera camera) {
        setCamera(camera);
        try {
            camera.setPreviewDisplay(mHolder);
        } catch (IOException exception) {
            Log.e(TAG, "IOException caused by setPreviewDisplay()", exception);
        }
        Camera.Parameters parameters = camera.getParameters();
        parameters.setPreviewSize(mPreviewSize.width, mPreviewSize.height);
        requestLayout();

        camera.setParameters(parameters);
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec)
    {
        final int width  = resolveSize(getSuggestedMinimumWidth(),  widthMeasureSpec);
        final int height = resolveSize(getSuggestedMinimumHeight(), heightMeasureSpec);
        setMeasuredDimension(width, height);

        if (mSupportedPreviewSizes != null) {
            mPreviewSize = getOptimalPreviewSize(mSupportedPreviewSizes, width, height);
        }
    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        if (changed && getChildCount() > 0) {
            final View child = getChildAt(0);

            int width = r - l;
            int height = b - t;

            int previewWidth = width;
            int previewHeight = height;
            if (mPreviewSize != null) {
                previewWidth = mPreviewSize.width;
                previewHeight = mPreviewSize.height;
                // kozawa add s
                width  = mPreviewSize.width;
                height = mPreviewSize.height;
                // kozawa add e
            }

            if (width * previewHeight > height * previewWidth) {
                final int scaledChildWidth = previewWidth * height / previewHeight;
                child.layout((width - scaledChildWidth) / 2, 0,
                        (width + scaledChildWidth) / 2, height);
            } else {
                final int scaledChildHeight = previewHeight * width / previewWidth;
                child.layout(0, (height - scaledChildHeight) / 2,
                        width, (height + scaledChildHeight) / 2);
            }

            Log.d(TAG, "previewWidth = " + previewWidth + ", previewHeight = " + previewHeight);
        }
    }

    public void surfaceCreated(SurfaceHolder holder)
    {
        try {
            if (mCamera != null) {
                mCamera.setPreviewDisplay(holder);

                (new Thread(ResultDetectorPreview.this)).start();

            }
        } catch (IOException exception) {
            Log.e(TAG, "IOException caused by setPreviewDisplay()", exception);
        }
    }

    public void surfaceDestroyed(SurfaceHolder holder)
    {
        if (mCamera != null) {
            mCamera.stopPreview();
        }
    }

    public void run() {
        mThreadRun = true;
        while (mThreadRun) {
            synchronized (this) {
                try {
                    this.wait();
                    processFrame(mFrame);
                    //Thread.sleep(500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    private Size getOptimalPreviewSize(List<Size> sizes, int w, int h) {
        final double ASPECT_TOLERANCE = 0.1;
        double targetRatio = (double) w / h;
        if (sizes == null) return null;

        Size optimalSize = null;
        double minDiff = Double.MAX_VALUE;

        int targetHeight = h;

        // Try to find an size match aspect ratio and size
        for (Size size : sizes) {
            double ratio = (double) size.width / size.height;
            if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE) continue;
            if (Math.abs(size.height - targetHeight) < minDiff) {
                optimalSize = size;
                minDiff = Math.abs(size.height - targetHeight);
            }
        }

        // Cannot find the one match the aspect ratio, ignore the requirement
        if (optimalSize == null) {
            minDiff = Double.MAX_VALUE;
            for (Size size : sizes) {
                if (Math.abs(size.height - targetHeight) < minDiff) {
                    optimalSize = size;
                    minDiff = Math.abs(size.height - targetHeight);
                }
            }
        }
        return optimalSize;
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h)
    {
        Camera.Parameters parameters = mCamera.getParameters();
        parameters.setPreviewSize(mPreviewSize.width, mPreviewSize.height);
        /*
        List<String> FocusModes = parameters.getSupportedFocusModes();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.GINGERBREAD){
            if (FocusModes.contains(Camera.Parameters.FOCUS_MODE_INFINITY)){
                parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_INFINITY);
            }
        }*/
        requestLayout();

        /* Now allocate the buffer */
        int size = parameters.getPreviewSize().width * parameters.getPreviewSize().height;
        size  = size * ImageFormat.getBitsPerPixel(parameters.getPreviewFormat()) / 8;
        mBuffer = new byte[size];
        /* The buffer where the current frame will be copied */
        mFrame = new byte [size];
        mCamera.addCallbackBuffer(mBuffer);

        mCamera.setParameters(parameters);
        mCamera.startPreview();
        new Thread() {
            @Override
            public void run() {
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                autoFocus();
            }
        }.start();
    }

    private long previousTimeMillis = 0;
    public String getFpf(long currentTimeMillis)
    {
        String fps_str = "0";
        if( previousTimeMillis == 0 ){
            previousTimeMillis = currentTimeMillis;
            return fps_str;
        }
        float fps = (1000f / (currentTimeMillis - previousTimeMillis));
        fps_str = String.format("%1$3.1f",fps);
        previousTimeMillis = currentTimeMillis;

        return fps_str;
    }

    // I[gtH[JX
    public void autoFocus(){
        if( mCamera != null ){
            mCamera.autoFocus( new Camera.AutoFocusCallback() {
                @Override
                public void onAutoFocus(boolean success, Camera camera) {
                }
            });
        }
    }
    // ʐ^BeitH[JXj
    public void takePicture(){
        if( mCamera != null ){
            mCamera.autoFocus( new Camera.AutoFocusCallback() {
                @Override
                public void onAutoFocus(boolean success, Camera camera) {
                    new Thread(new Runnable(){
                        public void run() {
                            try{
                                Thread.sleep(3000);
                            }catch(Exception e){}
                        };
                    }).start();
                    mCamera.stopPreview();
                    ((ResultDetectorActivity)mContext).saveImage();
                    mCamera.startPreview();
                }
            });
        }
    }
    // ʐ^BeitH[JXȂj
    public void takePictureWithoutFocus(){
        if( mCamera != null ){
            mCamera.stopPreview();
            ((ResultDetectorActivity)mContext).saveImage();
            mCamera.startPreview();
        }
    }
    public byte[] getCurrentYUV(){
        return mYUV;
    }

    private void processFrame(byte[] data)
    {
        if(mYUV==null) mYUV = new byte[data.length];
        mYUV = data;

        if(mPreviewSize==null) return;

        // ==================================================================
        // FPS ̍XVƕ`XVv -> S
        fps = getFpf(System.currentTimeMillis());
        ((ResultDetectorActivity)mContext).getPreviewOverlay().setFps( fps );
        handler.sendMessage(new Message());
        // FPS ̍XVƕ`XVv -> E
        // ==================================================================

        if(isOnlyPreview || inUIoperation) return;

        // ==================================================================
        // lCeBȕ -> S
        mGuideInfo = ((ResultDetectorActivity)mContext).getPreviewOverlay().getGuidePointInfo();

        int imgWidth    = mPreviewSize.width;
        int imgHeight   = mPreviewSize.height;
        int guidePosX   = mGuideInfo[0].x;
        int guidePosY   = mGuideInfo[0].y;
        int guideWidth  = mGuideInfo[1].x;
        int guideHeight = mGuideInfo[1].y;
        int[] imgSize   = {
            imgWidth,   imgHeight,
            guidePosX,  guidePosY,
            guideWidth, guideHeight
        };

        if( mHiInfo==null ) mHiInfo   = new HiInfo();
        if(sampleImg==null) sampleImg = new byte[64*96*40];
        if(tehai==null)     tehai     = new byte[14];
        if(dora==null)      dora      = new byte[10];
        if(nakihai==null)   nakihai   = new byte[16];
        if(nakiflag==null)  nakiflag  = new boolean[16];
        if(haiNum==null)    haiNum    = new int[3];
        if(tehaiImg==null)  tehaiImg  = new byte[64*96*14];
        if(doraImg==null)   doraImg   = new byte[64*96*10];
        if(nakiImg==null)   nakiImg   = new byte[64*96*16];
        if(edgeImg==null)   edgeImg   = new  int[guideWidth * guideHeight];

        int detect_result = JanScouterUtil.JanResultDetector(
            mYUV,       imgSize,
            mHiPointX,  mHiPointY,
            mHiWidth,   mHiHeight, mHiArray,
            nakiflag,
            haiNum,
            tehaiImg, doraImg, nakiImg,
            edgeImg );

        if(DebugFlg) Log.d(TAG, "JanResultDetector result : " + detect_result);
        mHiInfo.setDetectResult(detect_result);
        mHiInfo.setHiInfo(mHiPointX, mHiPointY, mHiWidth, mHiHeight,edgeImg.clone());

        // ------------------------------------------------------------------
        // Detect OK ̃[g
        if( detect_result > 6 )
        {
            mHiInfo.setHiArray(mHiArray);
            ((ResultDetectorActivity)mContext).getPreviewOverlay().setHiInfo(mHiInfo);

            @SuppressWarnings("unused")
            int matching_result = JanScouterUtil.JanResultMatching(
                    sampleImg,
                    tehai, dora, nakihai, nakiflag,
                    haiNum,
                    tehaiImg, doraImg, nakiImg,
                    edgeImg );

            boolean isTsumo     = false;
            boolean isReach     = false;
            boolean isIppatsu   = false;
            boolean isRinshan   = false;
            boolean isChankan   = false;
            boolean[] condition = { isTsumo,isReach,isIppatsu,isRinshan,isChankan };
            int[]   result    = new int[3];
            int point_result = JanScouterUtil.JanComputeResult(condition, tehai, dora, nakihai, nakiflag, result);
            if( point_result == AGARI_RESULT_NG )
            {
                // ĂȂꍇ
            }
            else if( point_result == AGARI_RESULT_OK )
            {
                // Ăꍇ
                //int fu    = result[0];
                //int han   = result[1];
                //int point = result[2];
            }
        }
        ((ResultDetectorActivity)mContext).getPreviewOverlay().setHiInfo(mHiInfo);


        // lCeBȕ -> E
        // ==================================================================

        if(isOnlyAnalyze || inUIoperation) return;

        // ==================================================================
        // cʗpAʗp Bitmap p -> S
        bmp = Bitmap.createBitmap(edgeImg, guideWidth, guideHeight, Config.RGB_565);
        //bmp = Bitmap.createBitmap(edgeImg, mPreviewSize.width, mPreviewSize.height, Config.RGB_565);
        Matrix matrix = new Matrix();
        float angle = ((ResultDetectorActivity)mContext).getOrientationMatrix(true);
        matrix.postRotate(angle);
        bmp = Bitmap.createBitmap(bmp, 0, 0, bmp.getWidth(), bmp.getHeight(), matrix, true);
        ((ResultDetectorActivity)mContext).getPreviewOverlay().setDecodeBitmap(bmp);
        // cʗpAʗp Bitmap p -> E
        // ==================================================================
    }

    // I[o[C̕`XV
    private final Handler handler = new Handler() {
        public void handleMessage(Message msg) {
            ((ResultDetectorActivity)mContext).invalidateOverlay();
        }
    };

    public void setIsUIoperation(boolean flg){
    	inUIoperation = flg;
    }

    public void waitPreview(long time){
        try {
			Thread.sleep(time);
        } catch (InterruptedException e) {
            e.printStackTrace();
		}
    }
}